@@ -24,9 +24,11 @@ from functools import partial
 from datetime import datetime, timedelta, timezone
 from collections import deque
 import asyncio
+import inspect
 import logging
 import ssl
 import re
+import sys
 logger = logging.getLogger('openleadr')
 
 
@@ -143,14 +145,17 @@ class OpenADRServer:
         await self.run()
 
     async def stop(self):
-        delayed_call_tasks = [task for task in asyncio.all_tasks() if task.get_name().startswith('DelayedCall')]
-        for task in delayed_call_tasks:
-            task.cancel()
+        if sys.version_info.minor >= 8:
+            delayed_call_tasks = [task for task in asyncio.all_tasks()
+                                  if task.get_name().startswith('DelayedCall')]
+            for task in delayed_call_tasks:
+                task.cancel()
         await self.app_runner.cleanup()
 
-    def add_event(self, ven_id, signal_name, signal_type, intervals, callback, targets=None,
-                  targets_by_type=None, target=None, market_context="oadr://unknown.context",
-                  notification_period=None, ramp_up_period=None, recovery_period=None):
+    def add_event(self, ven_id, signal_name, signal_type, intervals, callback=None, event_id=None,
+                  targets=None, targets_by_type=None, target=None, response_required='always',
+                  market_context="oadr://unknown.context", notification_period=None,
+                  ramp_up_period=None, recovery_period=None):
         """
         Convenience method to add an event with a single signal.
 
@@ -178,7 +183,11 @@ class OpenADRServer:
             return
         if not re.match(r"^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?", market_context):
             raise ValueError("The Market Context must be a valid URI.")
-        event_id = utils.generate_id()
+        event_id = event_id or utils.generate_id()
+
+        if response_required not in ('always', 'never'):
+            raise ValueError("'response_required' should be either 'always' or 'never'; "
+                             f"you provided '{response_required}'.")
 
         # Figure out the target for this Event
         if target is None and targets is None and targets_by_type is None:
@@ -200,6 +209,7 @@ class OpenADRServer:
                                            signal_type=signal_type,
                                            signal_id=utils.generate_id(),
                                            targets=targets)
+
         # Make sure the intervals carry timezone-aware timestamps
         for interval in intervals:
             if utils.getmember(interval, 'dtstart').tzinfo is None:
@@ -217,14 +227,12 @@ class OpenADRServer:
         event = objects.Event(active_period=active_period,
                               event_descriptor=event_descriptor,
                               event_signals=[event_signal],
-                              targets=targets)
-        if ven_id not in self.message_queues:
-            self.message_queues[ven_id] = deque()
-        self.message_queues[ven_id].append(event)
-        self.services['event_service'].pending_events[event_id] = (event, callback)
+                              targets=targets,
+                              response_required=response_required)
+        self.add_raw_event(ven_id=ven_id, event=event, callback=callback)
         return event_id
 
-    def add_raw_event(self, ven_id, event, callback):
+    def add_raw_event(self, ven_id, event, callback=None):
         """
         Add a new event to the queue for a specific VEN.
         :param str ven_id: The ven_id to which this event should be distributed.
@@ -233,11 +241,26 @@ class OpenADRServer:
         :param callable callback: A callback that will receive the opt status for this event.
                                   This callback receives ven_id, event_id, opt_type as its arguments.
         """
+        if utils.getmember(event, 'response_required') == 'always':
+            if callback is None:
+                logger.warning("You did not provide a 'callback', which means you won't know if the "
+                               "VEN will opt in or opt out of your event. You should consider adding "
+                               "a callback for this.")
+            elif not asyncio.isfuture(callback):
+                args = inspect.signature(callback).parameters
+                if not all(['ven_id' in args, 'event_id' in args, 'opt_type' in args]):
+                    raise ValueError("The 'callback' must have at least the following parameters: "
+                                     "'ven_id' (str), 'event_id' (str), 'opt_type' (str). Please fix "
+                                     "your 'callback' handler.")
+
         if ven_id not in self.message_queues:
             self.message_queues[ven_id] = deque()
         event_id = utils.getmember(utils.getmember(event, 'event_descriptor'), 'event_id')
         self.message_queues[ven_id].append(event)
-        self.services['event_service'].pending_events[event_id] = (event, callback)
+        if callback is not None:
+            self.services['event_service'].pending_events[event_id] = (event, callback)
+        if utils.getmember(event, 'response_required') == 'never':
+            self.services['event_service'].schedule_event_updates(ven_id, event)
         return event_id
 
     def add_handler(self, name, func):
@@ -259,5 +282,5 @@ class OpenADRServer:
                 self.services['poll_service'].polling_method = 'external'
                 self.services['event_service'].polling_method = 'external'
         else:
-            raise NameError(f"Unknown handler {name}. "
-                            f"Correct handler names are: {self._MAP.keys()}")
+            raise NameError(f"""Unknown handler '{name}'. """
+                            f"""Correct handler names are: '{"', '".join(self._MAP.keys())}'.""")
